import IORedis from 'ioredis'
import { Job, Queue, Worker } from 'bullmq'
import config from '../utilities/config'
import { Server as SocketServer } from 'socket.io'
import { TSocket } from '../utilities/types'
import { queueLogger } from '../utilities/logger'

class QueueManager {
  private connection!: IORedis
  private queue!: Queue
  private worker!: Worker
  private io!: SocketServer

  public async boot(io: SocketServer) {
    this.io = io

    this.connection = new IORedis(config.REDIS_URL, {
      maxRetriesPerRequest: null
    })

    try {
      await this.connection.ping()
      queueLogger.info('Successfully connected to Redis')
    } catch (error) {
      queueLogger.error('Failed to connect to Redis:', error)
      process.exit(1)
    }

    this.queue = new Queue('jobs')
    this.worker = new Worker('jobs', this.processJob.bind(this), {
      connection: this.connection,
      concurrency: 10000
    })

    this.worker.on('completed', (job) => {
      queueLogger.info(`Job ${job?.id} has completed`)
    })

    this.worker.on('failed', (job, err) => {
      queueLogger.error(`Job ${job?.id} failed with error: ${err}`)
    })

    queueLogger.info('Queue manager loaded')
  }

  private async processJob(job: Job) {
    const { jobClass, params, socketId } = job.data

    console.log('Processing job:', job)
    try {
      const JobClass = await import(`../jobs/${jobClass}`)

      if (!JobClass) {
        queueLogger.warn(`Job class not found: ${jobClass}`)
        return
      }

      console.log('Job class:', JobClass)

      console.log('Job class:', JobClass.name)

      const job = new JobClass(params)

      if (socketId && this.io) {
        const socket = this.io.sockets.sockets.get(socketId)
        if (socket) {
          await job.execute(this.io, socket)
        } else {
          queueLogger.warn(`Socket not found for job: ${socketId}`)
          await job.execute(this.io)
        }
      } else {
        await job.execute(this.io)
      }
    } catch (error: any) {
      queueLogger.error(`Error processing job: ${error.message}`)
    }
  }

  public async addToQueue(jobClass: any, params: any, socket?: TSocket) {
    const jobData = {
      jobClass: jobClass.name,
      params,
      socketId: socket?.id
    }

    await this.queue.add('job', jobData)
  }
}

export default new QueueManager()
